@@ -1,6 +1,10 @@
 import gc
+import itertools
+import operator
 import os
 import re
+import sys
+import traceback
 import types
 import warnings
 
@@ -10,6 +14,11 @@ try:
 except NameError:
     from sets import Set as set
 
+try:
+    frozenset = frozenset
+except NameError:
+    from sets import ImmutableSet as frozenset
+
 try:
     sorted = sorted
 except NameError:
@@ -60,6 +69,21 @@ except ImportError:
     def samefile(file1, file2):
         return os.path.realpath(file1) == os.path.realpath(file2)
 
+def compact(lst):
+    "Remove all occurences of None from the given list."
+    return [x for x in lst if x is not None]
+
+def counted(objects):
+    """Count how many times each object appears in a list and return
+    list of (object, count) tuples.
+
+    >>> counted(['a', 'b', 'c', 'a', 'b', 'a'])
+    [('a', 3), ('b', 2), ('c', 1)]
+    >>> counted([])
+    []
+    """
+    return [(obj, len(list(group))) for obj, group in groupby(sorted(objects))]
+
 def camelize(name):
     """Covert name into CamelCase.
 
@@ -74,7 +98,6 @@ def camelize(name):
         return match.group(1).upper()
     return re.sub(r'(?:^|_)(.)', upcase, name)
 
-
 def underscore(name):
     """Convert name into underscore_name.
 
@@ -100,6 +123,20 @@ def underscore(name):
     name = re.sub(r'([A-Z]+)', capitalize, name)
     return re.sub(r'([A-Z])', underscore, name)
 
+def pluralize(word, count):
+    """Depending on the counter, return a singular or a plural form of the
+    given word.
+
+    >>> pluralize("word", 1)
+    'one word'
+    >>> pluralize("word", 2)
+    '2 words'
+    """
+    if count == 1:
+        return "one %s" % word
+    else:
+        return "%d %ss" % (count, word)
+
 def read_file_contents(filename):
     fd = file(filename)
     contents = fd.read()
@@ -163,6 +200,12 @@ def rlistdir(path):
 def get_names(objects):
     return map(lambda c: c.name, objects)
 
+def map_values(function, dictionary):
+    new_dictionary = {}
+    for key, value in dictionary.iteritems():
+        new_dictionary[key] = function(value)
+    return new_dictionary
+
 class DirectoryException(Exception):
     pass
 
@@ -212,6 +255,36 @@ def findfirst(pred, seq):
         if pred(item):
             return item
 
+def flatten(lst):
+    """Flatten given list.
+
+    >>> flatten([[1, 2, 3], [4, 5], [6, 7], [8]])
+    [1, 2, 3, 4, 5, 6, 7, 8]
+    """
+    return list(itertools.chain(*lst))
+
+# :: [set] -> set
+def union(*sets):
+    """Return a union of all the given sets.
+    """
+    # Since 2.6 set.union accepts multiple input iterables.
+    if sys.version_info >= (2, 6):
+        return set.union(*sets)
+    else:
+        return reduce(operator.or_, sets, set())
+
+# :: dict, object -> object
+def key_for_value(dictionary, value):
+    """Return the first key of dictionary that maps to given value.
+
+    >>> key_for_value({'a': 1, 'b': 2}, 2)
+    'b'
+    >>> key_for_value({}, 1)
+    """
+    for k, v in dictionary.iteritems():
+        if v == value:
+            return k
+
 def contains_active_generator(frame):
     return bool(all_of_type(gc.get_referrers(frame), types.GeneratorType))
 
@@ -230,6 +303,24 @@ def compile_without_warnings(stmt):
 def quoted_block(text):
     return ''.join(["> %s" % line for line in text.splitlines(True)])
 
+def class_of(obj):
+    if hasattr(obj, "__class__"):
+        return obj.__class__
+    return type(obj)
+
+def class_name(obj):
+    return class_of(obj).__name__
+
+def module_name(obj):
+    return class_of(obj).__module__
+
+def module_path_to_name(module_path, newsep="_"):
+    return re.sub(r'(%s__init__)?\.py$' % re.escape(os.path.sep), '', module_path).\
+        replace(os.path.sep, newsep)
+
+def last_traceback():
+    return "".join(traceback.format_tb(sys.exc_info()[2]))
+
 # Regular expressions helpers.
 
 RePatternType = type(re.compile(''))
