@@ -1,7 +1,7 @@
 import unittest
 
 class TestSomeClass(unittest.TestCase):
-    def test_object_initialization(self):
+    def test___init__(self):
         assert True # implemented test case
 
     def test_some_method(self):
