@@ -9,8 +9,9 @@ from fixture import TempIO
 from nose.tools import assert_equal
 
 from pythoscope.generator import add_tests_to_project
-from pythoscope.logger import get_output, set_output
-from pythoscope.store import Function, ModuleNotFound, PointOfEntry, Project
+from pythoscope.logger import DEBUG, INFO, get_output, log, set_output
+from pythoscope.store import Execution, Function, ModuleNotFound, \
+    PointOfEntry, Project
 from pythoscope.util import quoted_block, read_file_contents, set
 
 
@@ -52,9 +53,11 @@ def assert_single_class(info, name):
     assert_length(info.classes, 1)
     assert_equal(name, info.classes[0].name)
 
-def assert_single_function(info, name):
+def assert_single_function(info, name, args=None):
     assert_length(info.functions, 1)
     assert_equal(name, info.functions[0].name)
+    if args is not None:
+        assert_equal(args, info.functions[0].args)
 
 def assert_equal_sets(collection1, collection2):
     assert_equal(set(collection1), set(collection2))
@@ -68,13 +71,17 @@ def assert_not_raises(exception, callable):
     except exception:
         assert False, "Exception %s has been raised." % exception
 
-def assert_instance(object, type):
-    assert isinstance(object, type), \
+def assert_instance(obj, objtype):
+    assert isinstance(obj, objtype), \
            "Expected object %r to be of type %r, it was of type %r instead." % \
-           (object, type, type(object))
+           (obj, objtype, type(obj))
 
-def assert_matches(regexp, string):
-    assert re.match(regexp, string), \
+def assert_matches(regexp, string, anywhere=False):
+    if anywhere:
+        match = re.search
+    else:
+        match = re.match
+    assert match(regexp, string), \
         "Expected\n%s\nto match r'%s', but it didn't." % (quoted_block(string), regexp)
 
 class PointOfEntryMock(PointOfEntry):
@@ -128,6 +135,9 @@ def TestableProject(more_modules=[], project_type=ProjectInDirectory):
     project["module"].objects = [Function("function")]
     return project
 
+def EmptyProjectExecution():
+    return Execution(EmptyProject())
+
 def get_test_module_contents(project):
     """Get contents of the first test module of a project.
     """
@@ -168,13 +178,21 @@ class CapturedLogger:
     """Capture all log output and make it available to test via
     _get_log_output() method.
     """
+    log_level = INFO
+
     def setUp(self):
         self._old_output = get_output()
+        self._old_level = log.level
         self.captured = StringIO()
         set_output(self.captured)
+        log.level = self.log_level
 
     def tearDown(self):
         set_output(self._old_output)
+        log.level = self._old_level
 
     def _get_log_output(self):
         return self.captured.getvalue()
+
+class CapturedDebugLogger(CapturedLogger):
+    log_level = DEBUG
