@@ -1,85 +1,111 @@
+import array
 import os
 import re
+import sets
+import sys
 import time
+import types
 
+from nose.exc import SkipTest
 from nose.tools import assert_equal, assert_not_equal, assert_raises
 
 from pythoscope.astvisitor import parse
-from pythoscope.generator import add_tests_to_project
-from pythoscope.store import Project, Class, Method, Function, \
+from pythoscope.generator import add_tests_to_project, constructor_as_string
+from pythoscope.serializer import ImmutableObject, UnknownObject
+from pythoscope.store import Project, Class, Function, Method, \
      ModuleNeedsAnalysis, ModuleSaveError, TestClass, TestMethod, \
-     MethodCall, FunctionCall, LiveObject, wrap_call_arguments, \
-     wrap_object, PointOfEntry, GeneratorObject, Value, Type
-from pythoscope.util import read_file_contents, get_last_modification_time
+     MethodCall, FunctionCall, UserObject, GeneratorObject
+from pythoscope.util import read_file_contents, get_last_modification_time, \
+    sorted
 
-from helper import assert_contains, assert_doesnt_contain, assert_length,\
-     generate_single_test_module, P, ProjectInDirectory, \
-     ProjectWithModules, TestableProject, assert_contains_once, \
-     PointOfEntryMock, get_test_cases, assert_equal_sets, CapturedLogger
+from helper import CapturedDebugLogger, CapturedLogger, P, \
+    ProjectInDirectory, ProjectWithModules, TestableProject, assert_contains, \
+    assert_contains_once, assert_doesnt_contain, assert_equal_sets, \
+    assert_length, assert_matches, generate_single_test_module, get_test_cases, \
+    EmptyProjectExecution, assert_equal_strings
 
 # Let nose know that those aren't test functions/classes.
 add_tests_to_project.__test__ = False
 TestClass.__test__ = False
 TestMethod.__test__ = False
 
-def create_method_call(method, input, output, call_type, poe):
+def create_method_call(method, args, output, call_type, execution):
+    sargs = execution.serialize_call_arguments(args)
     if call_type == 'output':
-        return MethodCall(method, wrap_call_arguments(input), output=wrap_object(output))
+        return MethodCall(method, sargs, output=execution.serialize(output))
     elif call_type == 'exception':
-        return MethodCall(method, wrap_call_arguments(input), exception=wrap_object(output()))
+        return MethodCall(method, sargs, exception=execution.serialize(output()))
     elif call_type == 'generator':
-        return GeneratorObject(12345, method, poe, wrap_call_arguments(input),
-                               map(wrap_object, output))
+        return GeneratorObject(method, sargs, map(execution.serialize, output))
 
 def ClassWithMethods(classname, methods, call_type='output'):
     """call_type has to be one of 'output', 'exception' or 'generator'.
     """
+    execution = EmptyProjectExecution()
     method_objects = []
     method_calls = []
-    poe = PointOfEntry(Project('.'), 'poe')
 
     for name, calls in methods:
         method = Method(name)
         method_objects.append(method)
-        for input, output in calls:
-            method_calls.append(create_method_call(method, input, output, call_type, poe))
+        for args, output in calls:
+            method_calls.append(create_method_call(method, args, output, call_type, execution))
 
     klass = Class(classname, methods=method_objects)
-    live_object = LiveObject(12345, klass, poe)
-    live_object.calls = method_calls
-    klass.add_live_object(live_object)
+    user_object = UserObject(None, klass)
+    user_object.calls = method_calls
+    klass.add_user_object(user_object)
 
     return klass
 
+def ClassWithInstanceWithoutReconstruction(name):
+    # Cheating a bit, as this class won't be the same as one defined in
+    # module.py.
+    return Class(name), types.ClassType(name, (object,), {})()
+
+def stable_serialize_call_arguments(execution, args):
+    serialized_args = {}
+    for key, value in sorted(args.iteritems()):
+        serialized_args[key] = execution.serialize(value)
+    return serialized_args
+
 def FunctionWithCalls(funcname, calls):
-    poe = PointOfEntryMock()
+    execution = EmptyProjectExecution()
     function = Function(funcname)
-    function.calls = [FunctionCall(poe, function, wrap_call_arguments(i), wrap_object(o)) for (i,o) in calls]
+    function.calls = [FunctionCall(function,
+                                   stable_serialize_call_arguments(execution, i),
+                                   execution.serialize(o)) for (i,o) in calls]
     return function
 
 def FunctionWithSingleCall(funcname, input, output):
     return FunctionWithCalls(funcname, [(input, output)])
 
 def FunctionWithExceptions(funcname, calls):
-    poe = PointOfEntryMock()
+    execution = EmptyProjectExecution()
     function = Function(funcname)
-    function.calls = [FunctionCall(poe, function, wrap_call_arguments(i), exception=wrap_object(e())) for (i,e) in calls]
+    function.calls = [FunctionCall(function,
+                                   execution.serialize_call_arguments(i),
+                                   exception=execution.serialize(e())) for (i,e) in calls]
     return function
 
 def FunctionWithSingleException(funcname, input, exception):
     return FunctionWithExceptions(funcname, [(input, exception)])
 
 def GeneratorWithYields(genname, input, yields):
-    poe = PointOfEntryMock()
+    execution = EmptyProjectExecution()
     generator = Function(genname, is_generator=True)
-    gobject = GeneratorObject(12345, generator, poe, wrap_call_arguments(input), map(wrap_object, yields))
+    gobject = GeneratorObject(generator,
+                              execution.serialize_call_arguments(input),
+                              map(execution.serialize, yields))
     generator.calls = [gobject]
     return generator
 
 def GeneratorWithSingleException(genname, input, exception):
-    poe = PointOfEntryMock()
+    execution = EmptyProjectExecution()
     generator = Function(genname, is_generator=True)
-    gobject = GeneratorObject(12345, generator, poe, wrap_call_arguments(input), exception=wrap_object(exception()))
+    gobject = GeneratorObject(generator,
+                              execution.serialize_call_arguments(input),
+                              exception=execution.serialize(exception()))
     generator.calls = [gobject]
     return generator
 
@@ -112,10 +138,10 @@ class TestGenerator:
         assert_contains(result, "def test_one_more(self):")
         assert_contains(result, "def test_a_function(self):")
 
-    def test_generates_nice_name_for_init_method(self):
+    def test_generates_conventional_name_for_init_method(self):
         objects = [Class('SomeClass', [Method('__init__')])]
         result = generate_single_test_module(objects=objects)
-        assert_contains(result, "def test_object_initialization(self):")
+        assert_contains(result, "def test___init__(self):")
 
     def test_ignores_empty_classes(self):
         result = generate_single_test_module(objects=[Class('SomeClass', [])])
@@ -143,10 +169,17 @@ class TestGenerator:
         assert_doesnt_contain(result, "from nose import SkipTest")
 
     def test_ignores_private_methods(self):
-        objects = [Class('SomeClass', map(Method, ['_semiprivate', '__private', '__eq__']))]
+        objects = [Class('SomeClass', map(Method, ['_semiprivate', '__private']))]
         result = generate_single_test_module(objects=objects)
         assert_doesnt_contain(result, "class TestSomeClass(unittest.TestCase):")
 
+    def test_doesnt_ignore_special_methods(self):
+        objects = [Class('SomeClass', map(Method, ['__eq__', '__init__']))]
+        result = generate_single_test_module(objects=objects)
+        assert_contains(result, "class TestSomeClass(unittest.TestCase):")
+        assert_contains(result, "def test___eq__(self):")
+        assert_contains(result, "def test___init__(self):")
+
     def test_ignores_private_functions(self):
         result = generate_single_test_module(objects=[Function('_function')])
         assert_doesnt_contain(result, "class")
@@ -259,6 +292,13 @@ class TestGenerator:
 
         assert_contains(result, "from module import Something")
 
+    def test_generates_imports_needed_for_arguments_of_init_methods(self):
+        klass = ClassWithMethods('Something', [('__init__', [({'fun': read_file_contents}, None)])])
+
+        result = generate_single_test_module(objects=[klass])
+
+        assert_contains(result, "from pythoscope.util import read_file_contents")
+
     def test_ignores_repeated_calls(self):
         objects = [FunctionWithCalls('square', 2*[({'x': 4}, 16)])]
 
@@ -300,12 +340,12 @@ class TestGenerator:
 
     def test_ignores_internal_object_calls(self):
         klass = ClassWithMethods('Something', [('method', [({'argument': 1}, 'result')])])
-        live_object = klass.live_objects[('poe', 12345)]
-        method_call = live_object.calls[0]
+        user_object = klass.user_objects[0]
+        method_call = user_object.calls[0]
 
-        subcall = MethodCall(Method('private'), {'argument': wrap_object(2)}, wrap_object(False))
+        subcall = MethodCall(Method('private'), {'argument': ImmutableObject(2)}, ImmutableObject(False))
         method_call.add_subcall(subcall)
-        live_object.add_call(subcall)
+        user_object.add_call(subcall)
 
         result = generate_single_test_module(objects=[klass])
 
@@ -376,15 +416,21 @@ class TestGenerator:
         assert_contains(result, "self.assertRaises(KeyError, lambda: something.method())")
 
     def test_generates_imports_for_user_defined_exceptions(self):
-        # Cheating a bit, as UserDefinedException class is not the same as one
-        # presumably defined in module.py.
-        class UserDefinedException(Exception): pass
-        function = Function('throw')
-        function.calls = [FunctionCall(None, function, {}, exception=Type(UserDefinedException()))]
+        klass = Class("UserDefinedException")
+        function = Function("throw")
+        function.calls = [FunctionCall(function, {}, exception=UserObject(None, klass))]
+
+        result = generate_single_test_module(objects=[function, klass])
+
+        assert_contains(result, "from module import UserDefinedException")
+
+    def test_doesnt_generate_imports_for_builtin_exceptions(self):
+        function = Function("throw")
+        function.calls = [FunctionCall(function, {}, exception=UnknownObject(Exception()))]
 
         result = generate_single_test_module(objects=[function])
 
-        assert_contains(result, "from test.test_generator import UserDefinedException")
+        assert_doesnt_contain(result, "import Exception")
 
     def test_generates_assert_equal_type_for_functions_returning_functions(self):
         objects = [FunctionWithSingleCall('higher', {}, lambda: 42)]
@@ -523,8 +569,8 @@ class TestGenerator:
         result = generate_single_test_module(objects=objects)
 
         assert_contains(result, "assert False")
-        assert_doesnt_contain(result, "assertEqual")
-        assert_doesnt_contain(result, "assertRaises")
+        assert_doesnt_contain(result, "  self.assertEqual")
+        assert_doesnt_contain(result, "  self.assertRaises")
 
     def test_handles_unicode_objects(self):
         objects = [FunctionWithSingleCall('characterize', {'x': u'\xf3'}, "o-acute")]
@@ -534,7 +580,7 @@ class TestGenerator:
         assert_contains(result, "def test_characterize_returns_oacute_for_unicode_string(self):")
         assert_contains(result, "self.assertEqual('o-acute', characterize(x=u'\\xf3'))")
 
-    def test_handles_pickable_function_objects(self):
+    def test_handles_localizable_function_objects(self):
         objects = [FunctionWithSingleCall('store', {'fun': read_file_contents}, None)]
 
         result = generate_single_test_module(objects=objects)
@@ -543,6 +589,124 @@ class TestGenerator:
         assert_contains(result, "def test_store_returns_None_for_read_file_contents_function(self):")
         assert_contains(result, "self.assertEqual(None, store(fun=read_file_contents))")
 
+    def test_handles_the_same_object_used_twice(self):
+        obj = []
+        objects = [FunctionWithSingleCall('compare', {'x': obj, 'y': obj}, True)]
+
+        result = generate_single_test_module(objects=objects)
+
+        assert_contains(result, "def test_compare_returns_true_for_x_equal_list_and_y_equal_list(self):")
+        assert_contains(result, "alist = []")
+        assert_contains(result, "self.assertEqual(True, compare(x=alist, y=alist))")
+
+    def test_handles_two_objects_used_many_times(self):
+        obj1, obj2 = [], []
+        objects = [FunctionWithSingleCall('four', {'w': obj1, 'x': obj2,
+                                                   'y': obj1, 'z': obj2}, False)]
+
+        result = generate_single_test_module(objects=objects)
+
+        assert_contains(result, "def test_four_returns_false_for_w_equal_list_and_x_equal_list_and_y_equal_list_and_z_equal_list(self):")
+        assert_contains(result, "alist1 = []")
+        assert_contains(result, "alist2 = []")
+        assert_contains(result, "self.assertEqual(False, four(w=alist1, x=alist2, y=alist1, z=alist2))")
+
+    def test_handles_many_objects_used_many_times(self):
+        obj1, obj2, obj3 = {}, {}, {}
+        objects = [FunctionWithSingleCall('six', {'a': obj1, 'b': obj2,
+                                                  'c': obj3, 'd': obj1,
+                                                  'e': obj2, 'f': obj3}, False)]
+
+        result = generate_single_test_module(objects=objects)
+
+        assert_contains(result, "def test_six_returns_false_for_a_equal_dict_and_b_equal_dict_and_c_equal_dict_and_d_equal_dict_and_e_equal_dict_and_f_equal_dict(self):")
+        assert_contains(result, "adict1 = {}")
+        assert_contains(result, "adict2 = {}")
+        assert_contains(result, "adict3 = {}")
+        assert_contains(result, "self.assertEqual(False, six(a=adict1, b=adict2, c=adict3, d=adict1, e=adict2, f=adict3))")
+
+    def test_uses_argument_name_when_argument_is_used_as_return_value(self):
+        obj = []
+        objects = [FunctionWithSingleCall('identity', {'x': obj}, obj)]
+
+        result = generate_single_test_module(objects=objects)
+
+        assert_contains(result, "def test_identity_returns_x_for_x_equal_list(self):")
+        assert_contains(result, "alist = []")
+        assert_contains(result, "self.assertEqual(alist, identity(x=alist))")
+
+    def test_handles_objects_that_depend_on_each_other(self):
+        inner = {}
+        outer = [inner]
+        objects = [FunctionWithSingleCall('contains', {'inner': inner, 'outer': outer}, True)]
+
+        result = generate_single_test_module(objects=objects)
+
+        assert_contains(result, "def test_contains_returns_true_for_inner_equal_dict_and_outer_equal_list(self):")
+        assert_contains(result, "adict = {}")
+        assert_contains(result, "self.assertEqual(True, contains(inner=adict, outer=[adict]))")
+
+    def test_handles_objects_which_setups_depend_on_each_other(self):
+        inner = []
+        outer = [inner]
+        objects = [FunctionWithSingleCall('mangle', {'a1': inner, 'a2': outer, 'a3': outer}, False)]
+
+        result = generate_single_test_module(objects=objects)
+
+        assert_contains(result, "def test_mangle_returns_false_for_a1_equal_list_and_a2_equal_list_and_a3_equal_list(self):")
+        assert_contains(result, "alist1 = []")
+        assert_contains(result, "alist2 = [alist1]")
+        assert_contains(result, "self.assertEqual(False, mangle(a1=alist1, a2=alist2, a3=alist2))")
+
+    def test_handles_reused_objects_in_method_calls(self):
+        alist = []
+        klass = ClassWithMethods('Doubler', [('double', [({'lst': alist}, (alist, alist))])])
+
+        result = generate_single_test_module(objects=[klass])
+
+        assert_contains(result, "def test_double_returns_tuple_for_list(self):")
+        assert_contains(result, "alist = []")
+        assert_contains(result, "doubler = Doubler()")
+        assert_contains(result, "self.assertEqual((alist, alist), doubler.double(lst=alist))")
+
+    def test_generates_sample_assertions_in_test_stubs_for_functions(self):
+        objects = [Function('something', args=['arg1', 'arg2', '*rest'])]
+        result = generate_single_test_module(template='nose', objects=objects)
+
+        assert_contains(result, "class TestSomething:")
+        assert_contains(result, "# assert_equal(expected, something(arg1, arg2, *rest))")
+        assert_contains(result, "raise SkipTest # TODO: implement your test here")
+
+    def test_generates_sample_setup_and_sample_assertions_in_test_stubs_for_classes_without_init(self):
+        objects = [Class('Something', [Method('method', args=['**kwds'])])]
+        result = generate_single_test_module(template='nose', objects=objects)
+
+        assert_contains(result, "class TestSomething:")
+        assert_contains(result, "def test_method(self):")
+        assert_contains(result, "# something = Something()")
+        assert_contains(result, "# assert_equal(expected, something.method(**kwds))")
+        assert_contains(result, "raise SkipTest # TODO: implement your test here")
+
+    def test_generates_sample_setup_and_sample_assertions_in_test_stubs_for_classes_with_init(self):
+        objects = [Class('SomethingElse', [Method('__init__', args=['self', 'arg'])])]
+        result = generate_single_test_module(template='nose', objects=objects)
+
+        assert_contains(result, "class TestSomethingElse:")
+        assert_contains(result, "def test___init__(self):")
+        assert_contains(result, "# something_else = SomethingElse(arg)")
+        assert_doesnt_contain(result, "# assert_equal(expected, something_else.__init__(arg))")
+        assert_contains(result, "raise SkipTest # TODO: implement your test here")
+
+    def test_generates_sample_setup_and_sample_assertions_in_test_stubs_for_classes_with_new(self):
+        objects = [Class('SomethingCompletelyDifferent', [Method('__new__', args=['self', 'x'])])]
+        result = generate_single_test_module(template='nose', objects=objects)
+
+        assert_contains(result, "class TestSomethingCompletelyDifferent:")
+        assert_contains(result, "def test___new__(self):")
+        assert_contains(result, "# something_completely_different = SomethingCompletelyDifferent(x)")
+        assert_doesnt_contain(result, "# assert_equal(expected, something_completely_different.__new__(x))")
+        assert_contains(result, "raise SkipTest # TODO: implement your test here")
+
 class TestGeneratorWithTestDirectoryAsFile:
     def setUp(self):
         self.project = TestableProject()
@@ -602,22 +766,28 @@ class TestGeneratorWithSingleModule:
         assert_length(self.project["test_other"].test_cases, 0)
 
     def test_comments_assertions_with_user_objects_that_cannot_be_constructed(self):
-        # Cheating a bit, as Something class is not the same as one defined in
-        # module.py.
-        class Something(object): pass
-        klass = Class("Something")
-        instance = Something()
+        klass, instance = ClassWithInstanceWithoutReconstruction("Something")
 
-        poe = PointOfEntryMock()
-        function = Function("nofun")
-        function.calls = [FunctionCall(poe, function, {'x': Value(instance)}, Value("something else"))]
+        function = FunctionWithSingleCall("nofun", {'x': instance}, "something else")
         self.project["module"].objects = [klass, function]
 
-        add_tests_to_project(self.project, ["module.py"], 'unittest', False)
+        add_tests_to_project(self.project, [self.module_path], 'unittest')
         result = self.project["test_module"].get_content()
 
         assert_contains(result, "def test_nofun_returns_something_else_for_something_instance(self):")
-        assert_contains(result, "# self.assertEqual('something else', nofun(x=%r))" % instance)
+        assert_contains(result, "# self.assertEqual('something else', nofun(x=<TODO: test.test_generator.Something>))")
+
+    def test_generates_type_assertions_for_calls_with_composite_objects_which_elements_cannot_be_constructed(self):
+        klass, instance = ClassWithInstanceWithoutReconstruction("Unspeakable")
+
+        function = FunctionWithSingleCall("morefun", {}, [instance])
+        self.project["module"].objects = [klass, function]
+
+        add_tests_to_project(self.project, [self.module_path], 'unittest')
+        result = self.project["test_module"].get_content()
+
+        assert_contains(result, "def test_morefun_returns_list(self):")
+        assert_contains(result, "self.assertEqual(list, type(morefun()))")
 
 class TestGeneratorMessages(CapturedLogger):
     def test_reports_each_module_it_generates_tests_for(self):
@@ -638,3 +808,63 @@ class TestGeneratorMessages(CapturedLogger):
                              "Adding generated TestSomeClass to tests/test_module.py.")
         assert_contains_once(self._get_log_output(),
                              "Adding generated TestSomeFunction to tests/test_module.py.")
+
+class TestGeneratorDebugMessages(CapturedDebugLogger):
+    def test_debug_output_includes_packages_and_module_names(self):
+        project = ProjectWithModules(["module.py"])
+        project["module"].objects = [Function('some_function')]
+
+        add_tests_to_project(project, ["module"], 'unittest')
+
+        assert_matches(r"\d+\.\d+ generator:\d+ INFO: Generating tests for module module.py.\n",
+                       self._get_log_output(), anywhere=True)
+        assert_matches(r"\d+\.\d+ generator\.adder:\d+ INFO: Adding generated TestSomeFunction to tests/test_module.py.\n",
+                       self._get_log_output(), anywhere=True)
+
+class TestConstructorAsString:
+    def setUp(self):
+        self.serialize = EmptyProjectExecution().serialize
+
+    def test_reconstructs_set_from_sets_module(self):
+        call_string = constructor_as_string(self.serialize(sets.Set([1, 2, 3])))
+
+        assert_equal_strings("Set([1, 2, 3])", call_string)
+        assert_equal_sets([("sets", "Set")], call_string.imports)
+
+    def test_reconstructs_immutable_set_from_sets_module(self):
+        call_string = constructor_as_string(self.serialize(sets.ImmutableSet([1, 2, 3])))
+
+        assert_equal_strings("ImmutableSet([1, 2, 3])", call_string)
+        assert_equal_sets([("sets", "ImmutableSet")], call_string.imports)
+
+    def test_reconstructs_builtin_set(self):
+        # Set builtin was added in Python 2.4.
+        if sys.version_info < (2, 4):
+            raise SkipTest
+
+        call_string = constructor_as_string(self.serialize(set([1, 2, 3])))
+
+        assert_equal_strings("set([1, 2, 3])", call_string)
+        assert_equal_sets([], call_string.imports)
+
+    def test_reconstructs_builtin_frozenset(self):
+        # Frozenset builtin was added in Python 2.4.
+        if sys.version_info < (2, 4):
+            raise SkipTest
+
+        call_string = constructor_as_string(self.serialize(frozenset([1, 2, 3])))
+
+        assert_equal_strings("frozenset([1, 2, 3])", call_string)
+        assert_equal_sets([], call_string.imports)
+
+    def test_reconstructs_integer_arrays(self):
+        call_string = constructor_as_string(self.serialize(array.array('I', [1, 2, 3, 4])))
+
+        assert_equal_strings("array.array('I', [1L, 2L, 3L, 4L])", call_string)
+        assert_equal_sets(['array'], call_string.imports)
+
+    def test_reconstructs_floating_point_arrays(self):
+        call_string = constructor_as_string(self.serialize(array.array('d', [1, 2, 3, 4])))
+
+        assert_equal_strings("array.array('d', [1.0, 2.0, 3.0, 4.0])", call_string)
+        assert_equal_sets(['array'], call_string.imports)
